<?php
/**
 * MegaCal API
 *
 * API wrapper for MegaCal WordPress plugins
 */

namespace MegaCal\Client;

class MegaCalAPI {

	private static $instance;

	const EVENT_REQUEST = 'event';
	const AUTH_REQUEST = 'auth';
	const PING_REQUEST = 'ping';
	const USER_REQUEST = 'user';

	private static $event_request;
	private static $auth_request;
	private static $ping_request;
	private static $user_request;

	private function __construct() {

		self::$event_request = new EventRequest();
		self::$auth_request = new AuthRequest();
		self::$user_request = new UserRequest();
		self::$ping_request = new PingRequest();

	}

	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new MegaCalAPI();
		}

		return self::$instance;

	}

	/**
	 *  Make an API request
	 *
	 * @param  string $type     	The type of request
	 * @param  string $action 		The request action
	 * @param  array  $args     	The request parameters
	 *
	 * @return MegaCalResponse 	The response object
	 */
	public static function request( $request_type, $action, $args = array() ) {

		$current_request = null;

		switch ( strtolower( $request_type ) ) {
		case self::EVENT_REQUEST:
			$current_request = self::$event_request;
			break;
		case self::AUTH_REQUEST:
			$current_request = self::$auth_request;
			break;
		case self::PING_REQUEST:
			$current_request = self::$ping_request;
			break;
		case self::USER_REQUEST:
			$current_request = self::$user_request;
			break;	
		default:
			throw new ApiException( 'Invalid request type: "' . $request_type . '"' );
			break;
		}

		if ( null === $current_request ) {
			throw new ApiException( 'Request was null' );
		}

		if ( false === method_exists( $current_request, $action ) ) {
			throw new ApiException( 'Invalid request action: "' . $action . '"' );
		}

		// finally call the method
		$response = false;

		if ( ! empty( $args ) ) {
			$response = $current_request->$action( $args );
		} else {
			$response = $current_request->$action();
		}

		return $response;

	}

}