<?php
/** Templating: Copy this file to mytheme/megabase-calendar/views/megacal-event-detail.php to override it. */

if ( ! defined( 'ABSPATH' ) ) 
	die( 'Not Allowed' ); // Exit if accessed directly

// Send a 404 if event does not exist or is not published
// TODO: Preview?
if( empty( $event ) || !$event->get_published() ) {
	
	global $wp_query;
	$wp_query->set_404();
	status_header( 404 );
	get_template_part( 404 ); 
	exit();

}

$settings = MegabaseCalendar::megacal_get_settings();

get_header( apply_filters( 'megacal_event_detail_header_name', null ) );

?>

<section class="mainContent main-content">

	<header style="margin-top:20px;">	
		<h2 class="h1" >Events</h2>	
	</header>

	<main id="site-content" class="megacal-events-integration">
		<article id="megacal-event-detail-<?php esc_attr_e( $event->get_id() ); ?>" class="page megacal-event entry">
			
			<div class="megaContent">

				<?php 
					$event_img = $event->get_image_url(); 
					if( !empty( $event_img ) ):
				?>
					<div class="megacal-event-image megacal-event-content-section">
						<img src="<?php echo esc_url( $event_img ); ?>" />
					</div>	
				<?php endif; ?>

				<h1 class="h2"><?php esc_html_e( $event->get_title() ); ?></h1>

				<div class="megacal-event-schedule-details megaContent megacal-event-content-section">
					<div class="megaMeta">
						<?php $event_datetime = new DateTimeImmutable( $event->get_event_date() ); ?>

						<p class="eventDatep">
							<span class="megacal-event-date"><?php esc_html_e( $event_datetime->format( 'l, F jS, Y' ) ); ?></span> 
						</p>

						<?php 
						/* Time */
						if( 
							!empty( $event->get_start_time() ) 
							|| !empty( $event->get_end_time() ) 
						): ?>
							<p class="eventTimep">
						<?php endif; ?>
						
						<?php $fmt = $this->megacal_get_time_fmt(); ?>
							<?php if( !empty( $event->get_start_time() ) ): ?>
								<?php esc_html_e( date( $fmt, strtotime( $event->get_start_time() ) ) ); ?> 
							<?php endif; ?>
							
							<?php if( !empty( $event->get_end_time() ) ): ?>
								- <?php esc_html_e( date( $fmt, strtotime( $event->get_end_time() ) ) ); ?>
							<?php endif; ?>

						<?php if( 
							!empty( $event->get_start_time() ) 
							|| !empty( $event->get_end_time() ) 
						): ?>
							</p>
						<?php endif; ?>				
					
						<!-- <span class="megacal-event-time"> -->
							<?php //esc_html_e( $this->megacal_get_event_time_string( $event ) ); ?>
						<!-- </span> -->

					</div>

				</div>

				<div class="megacal-event-venue-details megacal-event-content-section">

					<?php $venue = $event->get_venue(); ?>

					<h4 style="margin-bottom:4px;"><?php echo apply_filters( 'megacal_event_detail_venue_heading', 'Venue' ); ?></h4>
					<p class="megacal-event-venue-name"><?php esc_html_e( $venue->get_name() ); ?>
					<br><?php esc_html_e( $venue->get_location() ); ?></p>

					<?php if( $event->get_event_category()->get_name() !== MEGACAL_DEFAULT_CATEGORY_NAME ): ?>
						<p class="event-category event-category-<?php esc_attr_e( $event->get_event_category()->get_id() ); ?>"><?php esc_html_e( $event->get_event_category()->get_name() ); ?></p>
					<?php endif; ?>

				</div>

				<div class="megacal-event-details megacal-event-content-section">
					
					<?php if( !empty( $event->get_price_details() ) ): ?>
						<p class="megacal-event-cost">
							<?php echo apply_filters( 'megacal_event_detail_cost_label', 'Cost: ' ); ?>
							<?php esc_html_e( $event->get_price_details() ); ?>
						</p>
					<?php endif; ?>

					<?php if( !empty( $event->get_organizer() ) ): ?>
						<div class="megacal-event-orgaizers">
							<?php echo $this->megacal_esc_wysiwyg( $event->get_organizer() ); ?>
						</div>
					<?php endif; ?>

					<?php if( !empty( $event->get_description() ) ): ?>
						<div class="megacal-event-description">
							<?php echo $this->megacal_esc_wysiwyg( $event->get_description() ); ?>
						</div>
					<?php endif; ?>

				</div>

				<div class="megacal-event-links megacal-event-content-section">

					<?php $classes = apply_filters( 'megacal_event_detail_control_bar_classes', 'moveCenter' ); ?>
					<ul class="controlbar cf <?php esc_attr_e( $classes ); ?>">
						<?php if( !empty( $event->get_ticket_url() ) ): ?>
							<li>
								<a href="<?php echo esc_url( $event->get_ticket_url(), array( 'http', 'https' ) ); ?>" class="blueBtn greenBtn button  TicketLink" target="_blank" title="More information at <?php esc_attr_e( $event->get_ticket_url() ); ?>">
									<span><?php esc_html_e( $settings['megacal_event_url_label'] ); ?></span>
								</a>
							</li>
						<?php endif; ?>

						<?php if( !empty( $event->get_facebook_url() ) ): ?>
							<li>
								<a href="<?php echo esc_url( $event->get_facebook_url(), array( 'http', 'https' ) ); ?>" class="ticketsLink btn fbLinkWide" title="RSVP on Facebook" target="_blank"><i class="fui-facebook"></i>RSVP</a>
							</li>
						<?php endif; ?>
					</ul><!-- /.controlbar -->

				</div>

			</div>

		</article>
	</main>

</section>

<?php get_footer( apply_filters( 'megacal_event_detail_footer_name', null ) ); ?>