<?php
/** Templating: Copy this file to mytheme/megabase-calendar/views/megacal-full-events-list.php to override it. */

	if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

	if( empty( $response ) ) {
		return;
	}

	$events = $response->get_events();
?>

<?php if( !empty( $events ) && !( $events instanceof WP_Error ) ): ?>
	<?php foreach($events as $event): ?>

		<?php

			$date = new DateTimeImmutable( $event->get_event_date() );

			include trailingslashit( MEGACAL_PLUGIN_DIR ) . 'includes/partials/part-month-label.php';

			$venue = $event->get_venue();

			if( !empty( $event->get_event_date() ) ):
				
				$event_date = $event->get_event_date();
				?>

				<li id="event-<?php esc_attr_e( $event->get_id() ); ?>" class="summaryContent listEvent listItem cf">

					<?php if( !empty( $event->get_description() ) ): ?>
						<?php $description_class = ( $descrip ) ? 'expanded' : 'collapsed'; ?> 
						<div class="summaryToggle <?php esc_attr_e( $description_class ); ?>">
							<i class="fui-plus-circle" title="Click to Show More"></i>
						</div>
					<?php endif; ?>

					<div class="leftDate">
						<div class="megaDateRegion">
							<h3 class="megaDate" title="<?php echo date('D M d, Y', strtotime( $event_date )); ?>">
								<span class="monthName dayNameAbb"><?php echo date('D', strtotime( $event_date )); ?></span>
								<span class="monthName"><?php echo date('M', strtotime( $event_date )); ?></span>
								<span class="dayNum"><?php echo date('d', strtotime( $event_date )); ?></span>
								<span class="yearNum"><?php echo date('Y', strtotime( $event_date )); ?></span>
							</h3>
						</div>
					</div>

					<div class="eventPosterArea">
						<?php $ticket_link = !empty( $event->get_ticket_url() ) ? esc_url( $event->get_ticket_url(), array( 'http', 'https' ) ) : false; ?>

						<?php if( !empty( $ticket_link ) ): ?>
							<a href="<?php echo esc_url( $ticket_link, array( 'http', 'https' ) ); ?>" target="_blank" title="More information at <?php esc_attr_e( $ticket_link ); ?>">
						<?php endif; ?>

						<!-- If a Poster or Event Thumbnail Exists -->
						<?php if( !empty( $event->get_image_url() ) ): ?>
								<?php $thumbnail_src = $event->get_image_url(); ?>
								<img src="<?php echo esc_url( $thumbnail_src ); ?>" class="eventImg imgShadow alignleft" alt="<?php esc_attr_e( $event->get_title() ); ?>" />
						<?php else: ?>
							<div>
								<img src="<?php echo esc_url( megacal_get_default_event_image_path(), array( 'http', 'https' ) ); ?>" class="artistReplace alignleft defaultPic" />
							</div>
						<?php endif; ?>

						<?php if( !empty( $ticket_link ) ): ?>
							</a>
						<?php endif; ?>
					</div>

					<div class="listingBody">
						<h3 class="titleofevent">
							<a href="<?php echo site_url( trailingslashit( $settings['megacal_events_url'] ) . esc_attr( $event->get_id() ) ); ?>" class="showDetailsLink" title="View Details">
								<?php 
									if( !empty( $event->get_title() ) ) { 
										esc_html_e( $event->get_title() ); 
									} 
								?>
								<span class="atText">at</span> <?php esc_html_e( $venue->get_name() ); ?>
							</a>
						</h3><!-- /.titleofevent -->

						
						<div class="eventInfoArea">
							<div class="eventInfoWide">
								<p class="megacal-venue-location"><?php esc_html_e( $venue->get_location() ); ?></p>

								<p class="eventLogistics">
									<?php esc_html_e( $this->megacal_get_event_time_string( $event ) ); ?> 
									
									<?php if( !empty( $event->get_price_details() ) ): ?>
										<span class="sep">&bull;</span> Cost: <?php esc_html_e( $event->get_price_details() ); ?>
									<?php endif; ?>
								</p>

								<?php if( $event->get_event_category()->get_name() !== MEGACAL_DEFAULT_CATEGORY_NAME ): ?>
									<p class="event-category event-category-<?php esc_attr_e( $event->get_event_category()->get_id() ); ?>"><?php esc_html_e( $event->get_event_category()->get_name() ); ?></p>
								<?php endif; ?>

								<p class="marginBottom">
									<a href="<?php echo site_url( trailingslashit( $settings['megacal_events_url'] ) . esc_attr( $event->get_id() ) ); ?>" class="inconViewLink showDetailsLink" title="View Details">View Event</a>
								</p>

								<?php if( !empty( $event->get_description() ) ): ?>
									<div class="fullInfo" <?php if( $descrip == false ): ?>style="display: none;"<?php endif; ?>>
										<div class="eventDescription">
											<?php echo $this->megacal_esc_wysiwyg( $event->get_description() ); ?>
										</div>
									</div><!-- /.fullInfo -->
								<?php endif; ?>

								<?php if( !empty( $event->get_organizer() ) ): ?>
									<div class="eventDescription">
										<?php echo $this->megacal_esc_wysiwyg( $event->get_organizer() ); ?>
									</div><!-- /.fullInfo -->
								<?php endif; ?>

							</div><!-- /.eventInfoWide -->
						</div><!-- /.eventInfoArea -->
					</div><!-- /.listingBody -->

					<?php
						$className = "";

						if($buttons == 'left')
							$className = "moveLeft";
						elseif($buttons == 'right')
							$className = "moveRight";
						elseif($buttons == 'center')
							$className = "moveCenter";
					?>
					<ul class="controlbar cf <?php esc_attr_e( $className ); ?>">
						<?php if( !empty( $event->get_ticket_url() ) ): ?>
							<li>
								<a href="<?php echo esc_url( $event->get_ticket_url(), array( 'http', 'https' ) ); ?>" class="greenBtn TicketLink" target="_blank" title="More information at <?php esc_attr_e( $event->get_ticket_url() ); ?>">
								<span><?php esc_html_e( $settings['megacal_event_url_label'] ); ?></span>
								</a>
							</li>
						<?php endif; ?>

						<?php if( !empty( $event->get_facebook_url() ) ): ?>
							<li>
								<a href="<?php echo esc_url( $event->get_facebook_url(), array( 'http', 'https' ) ); ?>" class="ticketsLink btn fbLinkWide" title="RSVP on Facebook" target="_blank"><i class="fui-facebook"></i>RSVP</a>
							</li>
						<?php endif; ?>
					</ul><!-- /.controlbar -->
				</li>
				<?php 

			endif; 
		?>
	<?php endforeach; ?>

	<?php include trailingslashit( MEGACAL_PLUGIN_DIR ) . 'includes/partials/part-list-total-pages.php'; ?>

<?php else: ?>

	<?php
		if( !empty( $start_date ) )  {
			$date = new DateTimeImmutable( $start_date );
			include trailingslashit( MEGACAL_PLUGIN_DIR ) . 'includes/partials/part-month-label.php';
		}
	?>

	<li id="event-none" class="summaryContent listEvent listItem cf">
		<p><?php echo esc_html( $settings['megacal_no_event_msg'] ); ?></p>

		<?php if( $events instanceof WP_Error && is_user_logged_in() ): ?>
			<p><?php echo esc_html( $events->get_error_message() ); ?></p>	
		<?php endif; ?>
	</li>
<?php endif; ?>

<?php
